
/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */



package sfclasses;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import com.sun.java.swing.*;

/**
 * Frame for Firewall Administration.<br>
 * This is the GUI class providing access to all firewall configuration
 * and administration functions. The frame is constructed by the userAction
 * function of the host class when the user double clicks on a firewall
 * object.
 * @see Host#userAction
 * @version 1.0 16 Dec 1996
 * @author Roland E. Schmid
 */

public class FirewallAction
	extends Frame
	implements Runnable, ActionListener {

  /**
   * Initialize firewall action frame for host fw.
   * @param fw Firewall
   */
  public FirewallAction(Host fw) {
    firewall = fw;
    setTitle(firewall.getObjectID());
    setLayout(new BorderLayout());

    // create menus
    mbar = new MenuBar();

    FileMenu = new Menu("File");
    FileMenu.add(FileClose = new MenuItem("Close"));
    mbar.add(FileMenu);

    AdminMenu = new Menu("Administration");
    Menu ac = new Menu("Active Configuration");
    ac.add(AdminLoadC = new MenuItem("Load Configuration"));
    ac.add(AdminShowC = new MenuItem("Show Configuration"));
    ac.add(AdminShowV = new MenuItem("Show Variables"));
    AdminMenu.add(ac);

    Menu tc = new Menu("TCP Connections");
    tc.add(TCPShow = new MenuItem("Show Connections"));
    tc.add(TCPKill = new MenuItem("Kill Connection"));
    AdminMenu.add(tc);
    AdminMenu.add(AdminReset = new MenuItem("Reset Usage Counter"));

    Menu lg = new Menu("Logfiles");
    lg.add(LogMain = new MenuItem("Main"));
    lg.add(LogSpy = new MenuItem("Spy"));
    lg.add(LogReport = new MenuItem("Report"));
    AdminMenu.add(lg);
    mbar.add(AdminMenu);

    ConfigMenu = new Menu("Configuration");
    ConfigMenu.add(ConfGenerate = new MenuItem("Generate Config File"));
    ConfigMenu.add(ConfRestart = new MenuItem("Restart Firewall"));
    mbar.add(ConfigMenu);

    HelpMenu = new Menu("Help");
    HelpMenu.add(HelpGuide = new MenuItem("User's Guide"));
    HelpMenu.add(HelpAbout = new MenuItem("About"));
    mbar.add(HelpMenu);
    mbar.setHelpMenu(HelpMenu);

	 FileClose.addActionListener(this);
	 AdminLoadC.addActionListener(this);
	 AdminShowC.addActionListener(this);
	 AdminShowV.addActionListener(this);
	 AdminReset.addActionListener(this);
	 TCPShow.addActionListener(this);
	 TCPKill.addActionListener(this);
	 LogMain.addActionListener(this);
	 LogSpy.addActionListener(this);
	 LogReport.addActionListener(this);
	 ConfGenerate.addActionListener(this);
	 ConfRestart.addActionListener(this);
	 HelpGuide.addActionListener(this);
	 HelpAbout.addActionListener(this);
	 
    setMenuBar(mbar); 
	 addWindowListener(new FAAdapter());
  }

	 // The Menus
	 MenuItem FileClose;
	 MenuItem AdminLoadC, AdminShowC, AdminShowV, AdminReset;
	 MenuItem TCPShow, TCPKill;
	 MenuItem ConfGenerate, ConfRestart;
	 MenuItem LogMain, LogSpy, LogReport;
	 MenuItem HelpGuide, HelpAbout;


  class FAAdapter extends WindowAdapter {
    public void WindowClosing(WindowEvent we) {
      if (isEnabled()) {
        if (tcpPanelAdded) {
          tcpThread.stop();
          remove(tcpPanel);
          tcpPanelAdded = false;
        }
        setVisible(false);
      }
		}
  } // class FAAdapter


  public void actionPerformed(ActionEvent ae) {
  Object source = ae.getSource();

      if (!isEnabled())
        return;

		// Process "Close"
      if (source == FirewallAction.FileClose) {
        if (tcpPanelAdded) 
        if (tcpPanelAdded) {
          tcpThread.stop();
          remove(tcpPanel);
          tcpPanelAdded = false;
        }
        setVisible(false);
        return;
      }

		// Process "Load Configuration"
      else if (source == AdminLoadC) {
        if (rulePanelAdded) {
          remove(rulePanel);
          rulePanelAdded = false;
          validate();
        }
        if (tcpPanelAdded) {
          tcpThread.stop();
          remove(tcpPanel);
          tcpPanelAdded = false;
          validate();
        }
        actionCommand = LOAD_CONFIG;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }

		// Process "Show Configuration"
      else if (source == AdminShowC) {
        if (tcpPanelAdded) {
          tcpThread.stop();
          remove(tcpPanel);
          tcpPanelAdded = false;
          validate();
        }
        if (rulePanelAdded)
          return;
        if (configLoaded) {
          add("Center", rulePanel);
          validate();
          rulePanelAdded = true;
          return;
        }
        actionCommand = LOAD_CONFIG;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }


		// Process "Show Variables"
      else if (source == AdminShowV) {
        if (!configLoaded) {
          UserDialog.ErrorBox("Configuration not loaded!");
          return;
        }
        if (varShowing) {
          varDialog.setVisible(true);
          return;
        }
        varDialog = new VariableDialog(this, firewall);
        varDialog.setVisible(true);
        return; 
      }


		// Process "Show Connections"
      else if (source == TCPShow) {
        if (tcpPanelAdded) {
          tcpThread.stop();
          remove(tcpPanel);
          tcpPanelAdded = false;
          validate();
        }
        if (rulePanelAdded) {
          remove(rulePanel);
          rulePanelAdded = false;
          validate();
        }
        actionCommand = LOAD_TCP;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        tcpThread = actionThread;
        actionThread.start();
        return; 
      }


		// Process "Kill Connection"
      else if (source == TCPKill) {
        actionCommand = KILL_TCP;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }


		// Process "Reset Usage Counter"
      else if (source == AdminReset) {
        actionCommand = RESET_USAGE;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }


		// Process Log Submenus
      else if ((source == LogMain) || (source == LogSpy) || (source == LogReport)) {
        filename = new String(ManageControl.custom.getProperty("LOG_DIR"));
        if (source == LogMain)
          filename += ManageControl.custom.getProperty("LOG");
        if (source == LogSpy)
          filename += ManageControl.custom.getProperty("SPY_LOG");
        if (source == LogReport)
          filename += ManageControl.custom.getProperty("REPORT");
        actionCommand = LOAD_LOG;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }

		// Process "Generate Config File"
      else if (source == ConfGenerate) {
        actionCommand = GENERATE_CONFIG;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }

		// Process "Restart Firewall"
      else if (source == ConfRestart) {
        actionCommand = RECONFIG;
        setEnabled(false);
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        actionThread = new Thread(this);
        actionThread.start();
        return;
      }
		
		// Process "About"
      else if (source == HelpAbout) {
        AboutDialog ab = new AboutDialog(this);
        ab.setVisible(true);
        return;
      }

		// Process "User's Guide"
      else if (source == HelpGuide) {

			UserDialog.NoticeBox("Documentation only available as a PostScript file.");
      return;
      }
  } // actionPerformed

  // thread

  private String filename;
  private int actionCommand = 0;
  private static final int GENERATE_CONFIG = 1; // generate firewall configuration file
  private static final int RECONFIG        = 2; // reconfigure firewall
  private static final int LOAD_CONFIG     = 3; // load the active configuration from the firewall
  private static final int LOAD_LOG        = 4; // load a log file from the firewall
  private static final int LOAD_TCP        = 5; // load the active TCP connections from the firewall
  private static final int KILL_TCP        = 6; // kill a TCP connection
  private static final int RESET_USAGE     = 7; // reset the usage counter

  /**
   * The administration functions are executed in a separate thread.
   * The run() method contains the thread code.
   */
  public void run() {
    BusyBox bb;
    boolean success;
		int answer;
    while (true) {
      switch (actionCommand) {
        case GENERATE_CONFIG:
          bb = new BusyBox("Generating configuration file. Please wait.");
          bb.start();
          success = firewall.generateConfig();
          bb.done();
          if (success) {
						answer = JOptionPane.showConfirmDialog(null,
							"Success. Do you want to restart the firewall?",
							"User Information",
							JOptionPane.YES_NO_OPTION);
            if (answer == JOptionPane.YES_OPTION) {
              actionCommand = RECONFIG;
              continue;
            }
          }
          else
            UserDialog.WarningBox("Error generating the configuration file! ("+Communicator.commError+")");
          break;
        case RECONFIG:
          bb = new BusyBox("Restarting firewall. Please wait.");
          bb.start();
          success = firewall.reconfig();
          bb.done();
          if (success)
            UserDialog.NoticeBox("Firewall restarted successfully.");
          else
            UserDialog.WarningBox("Error restarting the firewall! ("+Communicator.commError+")");
          break;


        case LOAD_CONFIG:
          success = firewall.loadDynamicConfig();
          if (success) {
            ruleCanvas = new ShowCanvas(firewall);
            ruleHeader = new ShowHeader(ruleCanvas);
            rulePanel = new ListPanel(ruleCanvas, ruleHeader);
            add("Center", rulePanel);
            validate();
            rulePanelAdded = true;
            configLoaded = true;
            if (varShowing)
              varDialog.refresh();
          }
          else
            UserDialog.ErrorBox("Cannot load configuration data! ("+Communicator.commError+")");
          break;


        case LOAD_LOG:
          bb = new BusyBox("Loading log file. Please wait.");
          bb.start();
          String temp = Communicator.getTempFile(firewall.HostAddresses.getFirstAddress(), filename);
          bb.done();
          if (temp == null) {
            UserDialog.ErrorBox("Cannot read "+filename+"! ("+Communicator.commError+")");
            break;
          }

					Frame f = new TextView(temp);
          break;


        case LOAD_TCP:
          boolean firstRun = true;
          TcpConnection oldsel = null;
          while (true) { 
            success = firewall.loadTcpConns();
            if (success) {
              if (!firstRun) {
                int share[] = tcpCanvas.getShare();
                oldsel = (TcpConnection)tcpCanvas.getSelectedEntry();
                remove(tcpPanel);
                tcpCanvas = new TcpCanvas(firewall, share);
              }
              else
                tcpCanvas = new TcpCanvas(firewall);
              tcpHeader = new TcpHeader(tcpCanvas, firewall);
              tcpPanel = new ListPanel(tcpCanvas, tcpHeader);
              add("Center", tcpPanel);
              validate();
              if (oldsel != null) {
                int oldid = oldsel.id;
                Enumeration en = firewall.tcpconns.elements();
                while (en.hasMoreElements()) {
                  oldsel = (TcpConnection)en.nextElement();
                  if (oldsel.id == oldid) {
                    tcpCanvas.selectEntry(oldsel);
                    break;
                  }
                }
              }
              tcpPanelAdded = true;
            }
            if (!success) {
              if (!firstRun) {
                remove(tcpPanel);
                validate();
                tcpPanelAdded = false;
              }
              UserDialog.ErrorBox("Cannot load TCP connections! ("+Communicator.commError+")");
              break;
            }
            setEnabled(true);
        		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            actionCommand = 0;
            firstRun = false;
            try {
              Thread.sleep(10000); // reload every ten seconds
            }
            catch (InterruptedException ignore) {}
          } // while
          break;
        case KILL_TCP:
          TcpConnection tc = null;
          if (tcpPanelAdded) 
            tc = (TcpConnection)tcpCanvas.getSelectedEntry();
          if (!tcpPanelAdded || tc == null) {
            UserDialog.ErrorBox("No connection selected!");
            break;
          }
          tcpThread.stop();
					answer = JOptionPane.showConfirmDialog(null,
						"Killing connection ID "+tc.id, "User Information",
						JOptionPane.OK_CANCEL_OPTION);
          remove(tcpPanel);
          tcpPanelAdded = false;
          if (answer == JOptionPane.OK_OPTION) {
            if (!Communicator.killTcp(firewall.HostAddresses.getFirstAddress(),
                               tc.id, tc.key, tc.created_jiffies))
              UserDialog.ErrorBox("Error killing TCP connection! ("+Communicator.commError+")");
          }
          actionCommand = LOAD_TCP;
          tcpThread = actionThread;
          continue;
        case RESET_USAGE:
					answer = JOptionPane.showConfirmDialog(null,
						"Do you really want to reset the usage counter?",
						"Reset Usage Counters",
						JOptionPane.YES_NO_OPTION);
          if (answer == JOptionPane.YES_OPTION) {
            if (!Communicator.resetUsage(firewall.HostAddresses.getFirstAddress()))
              UserDialog.ErrorBox("Error resetting usage counter! ("+Communicator.commError+")");
            else 
              if (rulePanelAdded) {
                remove(rulePanel);
                rulePanelAdded = false;
                actionCommand = LOAD_CONFIG;
                continue;
              }
          }
          break;
      } // switch
      setEnabled(true);
      setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
      actionCommand = 0;
      return;
    }
  }

  /**
   * The varShowing variable indicates if the variable dialog is currently visible.
   */
  protected boolean varShowing = false;

  // private data

  private Thread actionThread, tcpThread;
  private Host firewall;
  private boolean rulePanelAdded = false;
  private boolean tcpPanelAdded = false;
  private boolean configLoaded = false;

  // GUI data
  private MenuBar mbar;
  private Menu FileMenu;
  private Menu AdminMenu;
  private Menu ConfigMenu;
  private Menu HelpMenu;
  private ListPanel rulePanel;
  private ShowCanvas ruleCanvas;
  private ShowHeader ruleHeader;
  private ListPanel tcpPanel;
  private TcpCanvas tcpCanvas;
  private TcpHeader tcpHeader;
  private VariableDialog varDialog;

} // FirewallAction

// Canvas for showing the active rules using the ListPanel
class ShowCanvas 
	extends ListCanvas {

  public ShowCanvas(Host fw) {
    super(fw.rules);
    firewall = fw;
    mgDomain = fw.mgDomain;
    column_no = 7;
    column_share = new int[column_no];
    column_share[0] = 10;
    column_share[1] = 15;
    column_share[2] = 16;
    column_share[3] = 20;
    column_share[4] = 20;
    column_share[5] = 11;
    column_share[6] =  8;
    column_size = new int[column_no];
    column_pos = new int[column_no];
  }

  protected int drawColumn(Graphics g, Object listObj, int col,
    int ypos, FontMetrics fm, int fontheight, int maxascent) {

    String str[];
    Rule r = (Rule)listObj;
    int objIdx = listEntries.indexOf(listObj);

    if (col >= 0) { // draw column
    
      if (g == null) {
        if (objIdx == 0 && col == 0)
          listText = new String[listEntries.size()][][];
        if (col == 0)
          listText[objIdx] = new String[column_no][];
        str = r.getString(col, firewall, mgDomain);
        if (col == 0)
          str[0] += String.valueOf(objIdx+1);
        listText[objIdx][col] = str;
        return str.length;
      }
      else
        str = listText[objIdx][col];

      if (ypos + str.length*fontheight - offset_y < 0)
        return str.length;

      int start = 0;
      if (col==0) { // add rule number
        start = 1;
        int imageWidth;
        if (r.active) {
          imageWidth = greenBallImage.getWidth(null);
          if (column_size[0] > imageWidth)
            g.drawImage(greenBallImage, column_pos[col]+1, ypos-offset_y+1, null);
        }
        else {
          imageWidth = redBallImage.getWidth(null);
          g.drawImage(redBallImage, column_pos[col]+1, ypos-offset_y+1, null);
        }
        g.drawString(clipString(str[0], column_size[col]-3-imageWidth, fm), 
          column_pos[col]+imageWidth+2, 
          ypos + maxascent - offset_y);
      }

      for (int i=start; i < str.length; i++) 
        g.drawString(clipString(str[i], column_size[col]-3, fm), 
          column_pos[col]+1, (ypos + maxascent + i*fontheight) - offset_y);

      return str.length;

    }
    else { // draw multi column text

      if (r.comment != null && r.comment.length() > 0) {
        if (g == null) 
          return 1;

        String cs = clipString("Description: "+r.comment, 
          canvas_width-column_size[0]-4, fm);
        g.clearRect(column_pos[1]+1, ypos - offset_y,
          fm.stringWidth(cs), fontheight);
        g.drawString(cs, column_pos[1]+1, 
          ypos + maxascent - offset_y);
        return 1;
      }
      else 
        return 0;
    }
  }

  private Host firewall;
  private ManageDomain mgDomain;
  private String listText[][][];

  private static Image greenBallImage = Utils.greenBall();
  private static Image redBallImage = Utils.redBall();

} // ShowCanvas

// Header bar for showing the active rules using the ListPanel
class ShowHeader extends ListHeader {

  public ShowHeader(ListCanvas listCanvas) {
    super(listCanvas);
    header = new String[listCanvas.column_no];
    header[0] = "Rule";
    header[1] = "Action";
    header[2] = "Protocol";
    header[3] = "From";
    header[4] = "To";
    header[5] = "Notification";
    header[6] = "Usage";
  }

} // ShowHeader

// Dialog box for showing the variables
class VariableDialog
	extends Dialog
	implements ActionListener {

  public VariableDialog(FirewallAction p, Host h) {
    super(p, "Variables on "+h.objectID, false);
    parent = p;
    host = h;

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.insets = new Insets(3,3,3,3);
    setLayout(gbl);

    constructString();
    varArea = new TextArea(varString, 20, 60);
    varArea.setEditable(false);
    varArea.setFont(new Font("Courier", Font.PLAIN, 12));
    Utils.add_component(this, varArea, gbl, gbc, 0, 0, 1, 1, 100, 100);

    Panel pa = new Panel();
    OKbutton = new Button("OK");
	 OKbutton.addActionListener(this);
    pa.add(OKbutton);
    Utils.add_component(this, pa, gbl, gbc, 0, 1, 1, 1, 0, 0);

    parent.varShowing = true;
    pack();

	 addWindowListener(new VDAdapter());
  }

  class VDAdapter extends WindowAdapter {
  	public void WindowClosing(WindowEvent we) {
      if (isEnabled())
        dispose();
      return;
	}
  } // VDAdapter

  public void actionPerformed(ActionEvent ae) {
    if (ae.getSource() == OKbutton) {
      dispose();
      parent.varShowing = false;
      return;
    }
  } // actionPerformed


  private void constructString() {
    StringBuffer sb = new StringBuffer();
    Date actDate = new Date();
    int len;

    for (int i=0; i < host.varnames.length; i++) {
      sb.append(host.varnames[i]);
      sb.append(" = ");
      if (host.vartimeout[i][0].after(actDate)) {
        sb.append(varval(i,0));
        sb.append("\n");
      }
      else
        sb.append("0\n");
      for (int j=1; j < host.varvalues[i].length; j++) {
        if (host.vartimeout[i][j].after(actDate)) {
          sb.append("  host ");
          len = sb.length();
          sb.append(Utils.printIP(host.varaddr[i][j]));
          while (sb.length() - len < 15)
            sb.append(' ');
          sb.append(" = ");
          sb.append(varval(i,j));
          sb.append("\n");
        }
      }
      sb.append("\n");
    }

    varString = sb.toString();
  }

  private String varval(int varnum, int entrynum) {
    StringBuffer sb = new StringBuffer();
    sb.append(Integer.toString(host.varvalues[varnum][entrynum]));
    if (host.vartimeout[varnum][entrynum].before(maxDate)) {
      String tstr = host.vartimeout[varnum][entrynum].toString();
      sb.append(", timeout "+tstr.substring(tstr.indexOf(':')-2,tstr.indexOf(':')+6));
    }
    return sb.toString();
  }

  public void refresh() {
    constructString();
    varArea.setText(varString);
  }

  private static Date maxDate = new Date((long)Integer.MAX_VALUE * 1000);

  private FirewallAction parent;
  private Host host;

  private String varString;
  private TextArea varArea;
  private Button OKbutton;

} // VariableDialog

// Canvas to display the active TCP connections using the ListPanel
class TcpCanvas
	extends ListCanvas {

  public TcpCanvas(Host fw) {
    super(fw.tcpconns);
    firewall = fw;
    mgDomain = fw.mgDomain;
    column_no = 7;
    column_share = new int[column_no];
    column_share[0] =  5;
    column_share[1] = 15;
    column_share[2] = 12;
    column_share[3] = 12;
    column_share[4] = 12;
    column_share[5] = 22;
    column_share[6] = 22;
    column_size = new int[column_no];
    column_pos = new int[column_no];
  }

  public TcpCanvas(Host fw, int share[]) {
    super(fw.tcpconns);
    firewall = fw;
    mgDomain = fw.mgDomain;
    column_no = 7;
    column_share = share;
    column_size = new int[column_no];
    column_pos = new int[column_no];
  }

  protected int drawColumn(Graphics g, Object listObj, int col,
    int ypos, FontMetrics fm, int fontheight, int maxascent) {

    String str;
    TcpConnection tc = (TcpConnection)listObj;
    int objIdx = listEntries.indexOf(listObj);
    Image stateBall;
    int imageWidth;

    if (col >= 0) { // draw column
    
      if (g == null) {
        if (objIdx == 0 && col == 0)
          listText = new String[listEntries.size()][];
        if (col == 0)
          listText[objIdx] = new String[column_no];
        str = tc.getString(col);
        listText[objIdx][col] = str;
        return 1;
      }
      else
        str = listText[objIdx][col];

      if (ypos + fontheight - offset_y < 0)
        return 1;

      if (col == 1) { // state
        switch (tc.stateColor()) {
          case TcpConnection.RED:
            stateBall = redBallImage;
            break;
          case TcpConnection.YELLOW:
            stateBall = yellowBallImage;
            break;
          case TcpConnection.GREEN:
            stateBall = greenBallImage;
            break;
          default:
            stateBall = null;
            break;
        }
        if (stateBall != null) {
          imageWidth = stateBall.getWidth(null);
          if (column_size[col] > imageWidth)
            g.drawImage(stateBall, column_pos[col]+1, ypos - offset_y + 1, null);
        }
        else
          imageWidth = 0;
        g.drawString(clipString(str, column_size[col]-3-imageWidth, fm),
          column_pos[col]+imageWidth+2, ypos + maxascent - offset_y);
      }
      else
        g.drawString(clipString(str, column_size[col]-3, fm), 
          column_pos[col]+1, ypos + maxascent - offset_y);

      return 1;

    }
    else // no multi column text
      return 0;
  }

  private Host firewall;
  private ManageDomain mgDomain;
  private String listText[][];

  private static Image greenBallImage = Utils.greenBall();
  private static Image redBallImage = Utils.redBall();
  private static Image yellowBallImage = Utils.yellowBall();

} // TcpCanvas

// Header bar for the TCP connection monitor
class TcpHeader extends ListHeader {

  public TcpHeader(ListCanvas listCanvas, Host firewall) {
    super(listCanvas);
    header = new String[listCanvas.column_no];
    header[0] = "ID";
    header[1] = "State "+Utils.timeString(firewall.actDate);
    header[2] = "Created";
    header[3] = "Last used";
    header[4] = "Timeout";
    header[5] = "Source";
    header[6] = "Destination";
  }

} // TcpHeader

