/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */



/*
 * This file is part of the SINUS firewall loadable driver module
 *
 * $Id: sf_masq.c,v 0.0.1 1999/08/18 15:08:59 skyper Rel $
 *
 */

/* #define SF_DEBUG */
/* #define SF_DEBUG_MASQ */
/* #define SF_DEBUG_RULES */

#include <linux/config.h>
#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS 1
#endif
#include <linux/modversions.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/icmp.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/ip.h>
#include <linux/malloc.h>
#include <linux/timer.h>
#include <net/ip.h>
#include <net/tcp.h>
#include <linux/igmp.h>
#include <asm/checksum.h>
#include <net/checksum.h>
#include "sf_kernel.h"
#include "sf_rc.h"
#include "sf_global.h"
#include "sf_filter.h"
#include "sf_control.h"
#include "sf_tcp.h"
#include "sf_icmp.h"

#define TCP_MASQ_EXPIRES 300
#define UDP_MASQ_EXPIRES 120
#define ICMP_MASQ_EXPIRES 10
#define PRINTIP(a) printk("%lu.%lu.%lu.%lu ", ntohl(a)>>24&0xFF,\
                          ntohl(a)>>16&0xFF, ntohl(a)>>8&0xFF, ntohl(a)&0xFF)
static struct sf_masq_list *masqlist = NULL;


/* This function creates an entry in the list of masqueraded
 * packets/connections, recalculates checksums and changes the source-adress */

void add_masq(struct iphdr *ip, struct sf_fw *masqrule) {
  struct sf_masq_list entry;
  struct sf_masq_list *ptr, *ptr2;
  struct icmphdr *icmp = (struct icmphdr *) ((char *) ip + ip->ihl * 4);
  struct tcphdr *tcp = (struct tcphdr *) ((char *) ip + ip->ihl * 4);
  struct udphdr *udp = (struct udphdr *) ((char *) ip + ip->ihl * 4);
  int len = ntohs(ip->tot_len) - (ip->ihl * 4);
  __u32 masq_saddr = masqrule->masq_saddr;
  /* __u32 masq_daddr = masqrule->masq_daddr; */
  
  ptr = &entry;
  
  /* Find the end of the queue */
  
  ptr->masq_saddr = masq_saddr;
  ptr->masq_daddr = ip->daddr; /* For now... future implementations will
                                * support dst-addr rewriting...        */
  
  switch (ip->protocol) {
  case IPPROTO_ICMP:
    ptr->protocol = IPPROTO_ICMP;
    ptr->prot.icmp.type = icmp->type;
    ptr->prot.icmp.code = icmp->code;
    ptr->prot.icmp.echoid = icmp->un.echo.id;
    ptr->real_saddr = ip->saddr;
    ptr->real_daddr = ip->daddr;
    
    masq_queue(ptr, ICMP_MASQ_EXPIRES);
    break;
    
  case IPPROTO_TCP:
    ptr->protocol = IPPROTO_TCP;
    ptr->prot.tcpudp.sport = tcp->source;
    ptr->prot.tcpudp.dport = tcp->dest;
    ptr->real_saddr = ip->saddr;
    ptr->real_daddr = ip->daddr;
    
    tcp->check = 0;
    tcp->check = csum_tcpudp_magic(ptr->masq_saddr, ip->daddr, len, IPPROTO_TCP,
				   csum_partial((char *) tcp, len, 0));
    if ((ptr2 = find_tcpudp_entry(ptr)) != NULL) {
      del_timer(&ptr2->expire);
      kill_masq_entry(ptr2); /* uh... ok, queueing/dequeueing isn't very */
    }                               /* but ok for now ;)  */
    
    masq_queue(ptr, TCP_MASQ_EXPIRES);
    break;
    
  case IPPROTO_UDP:
    ptr->protocol = IPPROTO_UDP;
    ptr->prot.tcpudp.sport = udp->source;
    ptr->prot.tcpudp.dport = udp->dest;
    ptr->real_saddr = ip->saddr;
    ptr->real_daddr = ip->daddr;
    
    udp->check = 0;
    udp->check = csum_tcpudp_magic(ptr->masq_saddr, ip->daddr, len, IPPROTO_UDP,
				   csum_partial((char *) udp, len, 0));
    if ((ptr2 = find_tcpudp_entry(ptr)) != NULL) {
      del_timer(&ptr2->expire);
      kill_masq_entry(ptr2);
    }
    
    masq_queue(ptr, UDP_MASQ_EXPIRES);
    break;
  }
  
  ip->saddr = ptr->masq_saddr; /* Change the sourceaddress of the packet */
  ip->check = 0;
  ip->check = ip_fast_csum((unsigned char *) ip, ip->ihl);
}

int de_masq(struct iphdr *ip) {
  struct sf_masq_list *ptr;
  struct icmphdr *icmp = (struct icmphdr *) ((char *) ip + ip->ihl * 4);
  struct tcphdr *tcp = (struct tcphdr *) ((char *) ip + ip->ihl * 4);
  struct udphdr *udp = (struct udphdr *) ((char *) ip + ip->ihl * 4);
  int len = ntohs(ip->tot_len) - (ip->ihl * 4);
  __u32 real_saddr;
  
  for (ptr = masqlist->next; ptr != NULL; ptr = ptr->next) {
    if ((ptr->protocol != ip->protocol) ||
        (ptr->real_daddr != ip->saddr) || (ip->daddr != ptr->masq_saddr))
      continue;
    
    /* an entry, which matches the packet, was found in the list */
    
    switch (ip->protocol) {
    case IPPROTO_ICMP:
      switch (icmp->type) {
      case ICMP_ECHOREPLY:
	if ( (ptr->prot.icmp.type == ICMP_ECHO) &&
	     ((icmp->un.echo.id) == (ptr->prot.icmp.echoid)) ) {
	  real_saddr = ptr->real_saddr;
	  del_timer(&ptr->expire);
	  kill_masq_entry(ptr);
	  goto masqentry_match;
	}
	continue;
      }
    case IPPROTO_TCP:
      if ((tcp->source == ptr->prot.tcpudp.dport) &&
	  (tcp->dest == ptr->prot.tcpudp.sport)) {
	tcp->check = 0;
	tcp->check = csum_tcpudp_magic(ip->saddr, ptr->real_saddr, len,
				       IPPROTO_TCP, csum_partial((char *) tcp, len, 0));
	
	real_saddr = ptr->real_saddr;
	goto masqentry_match;
      } else
	continue;
      break;
      
    case IPPROTO_UDP:
      if ((udp->source == ptr->prot.tcpudp.dport) &&
	  (udp->dest == ptr->prot.tcpudp.sport)) {
	udp->check = 0;
	udp->check = csum_tcpudp_magic(ip->saddr, ptr->real_saddr, len,
				       IPPROTO_UDP, csum_partial((char *) udp, len, 0));
	real_saddr = ptr->real_saddr;
	
	goto masqentry_match;
      } else
	continue;
    }
  }
  
  /* No suitable entry was found... */
  return 0;
  
 masqentry_match:
  ip->daddr = real_saddr;
  ip->check = 0;
  ip->check = ip_fast_csum((unsigned char *)ip, ip->ihl);
  return 1;
}

/* this function checks the rules if the packet should be masqueraded
 * and returns the rule-entry, which should be used...
 * This function is neccesary due to lack of a separate rulematch-function
 * We should consider taking out the checkrules of the sf_check_packet()
 * funktion and put it in a seperate one...                             */
struct sf_fw * masq_rule( struct iphdr *ip ) {
  struct sf_fw *rule;
  struct tcphdr * tcp = (struct tcphdr *) ((char *) ip + (ip->ihl * 4));
  for (rule = prio_rules; rule != NULL; rule = rule->fw_next) {
    if (rule->fw_rc != FW_MASQUERADE)
      continue;
    
    if (!((ip->protocol == rule->protocol) ||
	  (rule->fw_flags & SF_FW_PROT) == SF_FW_PROT_ALL))
      continue;
    if (rule->fw_src_cnt) {
      int match = 0;
      int port  = -1;
      if ((ip->protocol == IPPROTO_TCP) || (ip->protocol == IPPROTO_UDP))
	port = ntohs(tcp->source);
      match = sf_addr_match(rule->fw_src_idx, rule->fw_src_cnt,
			    ip->saddr, port, rule->fw_flags & SF_FW_SRC_NEG);
      if ((match && (rule->fw_flags & SF_FW_SRC_NEG))
	  || (!match && !(rule->fw_flags & SF_FW_SRC_NEG)))
	continue;
    }
    if (rule->fw_dst_cnt) {
      int match = 0;
      int port  = -1;
      
      if ((ip->protocol == IPPROTO_TCP) || (ip->protocol == IPPROTO_UDP))
	port = ntohs(tcp->dest);
      match = sf_addr_match(rule->fw_dst_idx, rule->fw_dst_cnt,
                            ip->daddr, port, rule->fw_flags & SF_FW_DST_NEG);
      if ((match && (rule->fw_flags & SF_FW_DST_NEG))
	  || (!match && !(rule->fw_flags & SF_FW_DST_NEG)))
	continue;
    }
    
    /* rule matched, returning address to masquerade with */
    return rule;
  }
  /* No masquerading-rule for this packet... */
  return NULL;
}

void kill_masq_entry( struct sf_masq_list *ptr ) {
#ifdef SF_DEBUG_MASQ
  printk("dequeueing: "); PRINTIP(ptr->real_saddr);
  printk(" -> "); PRINTIP(ptr->real_daddr);
  printk("prot: %d\n", ptr->protocol);
#endif
  if (ptr->next != NULL)
    ptr->next->prev = ptr->prev;
  ptr->prev->next = ptr->next;
  kfree(ptr);
}

void masq_queue ( struct sf_masq_list *entry, __u16 secs ) {
  struct sf_masq_list *ptr, *ptr2;
  if (secs == 0) {
    kill_masq_entry(entry);
  } else {
    for (ptr = masqlist; ptr->next != NULL; ptr = ptr->next) {
#ifdef SF_DEBUG_MASQ
      printk("queue: "); PRINTIP(ptr->real_saddr);
      printk(" -> "); PRINTIP(ptr->real_daddr);
      printk("prot: %d\n", ptr->protocol);
#endif
    }
    
    ptr->next = kmalloc(sizeof(struct sf_masq_list), GFP_ATOMIC);
    ptr2 = ptr;
    ptr = ptr->next;
    memcpy(ptr, entry, sizeof(struct sf_masq_list));
    ptr->prev = ptr2;
    
    ptr->next = NULL;
    
    ptr->expire.next = NULL;
    ptr->expire.prev = NULL;
    ptr->expire.function = (void *) kill_masq_entry;
    ptr->expire.expires = jiffies + HZ*secs;
    ptr->expire.data = (unsigned long) ptr;
    add_timer(&ptr->expire);
#ifdef SF_DEBUG_MASQ
    printk("queueing: "); PRINTIP(ptr->real_saddr);
    printk(" -> "); PRINTIP(ptr->real_daddr);
    printk("prot: %d\n", ptr->protocol);
#endif
  }
}

void sf_del_all_masq_timers( void ) {
  struct sf_masq_list *ptr, *ptr2;
  for (ptr = masqlist; ptr != NULL;) {
#ifdef SF_DEBUG_MASQ
    printk("dequeueing: "); PRINTIP(ptr->real_saddr);
    printk(" -> "); PRINTIP(ptr->real_daddr);
    printk("prot: %d\n", ptr->protocol);
#endif
    if (ptr->expire.expires != 0)
      del_timer(&ptr->expire);
    ptr2=ptr->next;
    kfree(ptr);
    ptr=ptr2;
  }
}

struct sf_masq_list * find_tcpudp_entry(struct sf_masq_list *entry) {
  struct sf_masq_list *ptr;
  
  for (ptr = masqlist->next; ptr != NULL; ptr = ptr->next) {
    if ((ptr->protocol != IPPROTO_TCP) && (ptr->protocol != IPPROTO_UDP))
      continue;
    
    if ((ptr->real_saddr        == entry->real_saddr       )  &&
	(ptr->real_daddr        == entry->real_daddr       )  &&
	(ptr->prot.tcpudp.sport == entry->prot.tcpudp.sport)  &&
	(ptr->prot.tcpudp.dport == entry->prot.tcpudp.dport)) {
      return ptr;
    }
  }
  return NULL;
}
